/*
 * pc300cfg.c	Cyclades-PC300(tm) configuration utility.
 *
 * Author:	Ivan Passos <ivan@cyclades.com>
 *
 * Copyright:	(c) 1999-2001 Cyclades Corp.
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License
 *      as published by the Free Software Foundation; either version
 *      2 of the License, or (at your option) any later version.
 *
 * $Log: pc300.c,v $
 * Revision 0.5 2001/03/02 daniela
 * Added support for the new SIOCGPC300CONF ioctl format.
 *
 * Revision 0.4 2000/11/08 ivan
 * Added support for unframed E1 and monitor mode.
 *
 * Revision 0.3 2000/09/27 ivan
 * Support for channel ranges (e.g. 1-15) for the "active_ch" parameter.
 *
 * Revision 0.2 1999/11/16 daniela
 * X.25 support.
 *
 * Revision 0.1 1999/10/06 ivan
 * Initial version.
 *
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <linux/netdevice.h>
#include <net/if_arp.h>
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <netdb.h>

#include "pc300.h"

#define	CMDSZ	10

int skfd = -1;

static void
pc300_show_if(char *cpc_name, pc300chconf_t *conf)
{
    printf("Interface: %s\n", cpc_name);
    printf("- Media:\t\t");
    switch(conf->media) {
	case LINE_RS232:
	    printf("RS232\n");
	    break;
	case LINE_V35:
	    printf("V.35\n");
	    break;
	case LINE_X21:
	    printf("X.21\n");
	    break;
	case LINE_T1:
	case LINE_E1:
	    if (conf->media == LINE_T1) {
		printf("T1\n");
		printf("  - Line Build Out:\t");
		switch(conf->lbo) {
		    case PC300_LBO_0_DB:
			printf("0 dB\n");
			break;
		    case PC300_LBO_7_5_DB:
			printf("7.5 dB\n");
			break;
		    case PC300_LBO_15_DB:
			printf("15 dB\n");
			break;
		    case PC300_LBO_22_5_DB:
			printf("22.5 dB\n");
			break;
		    default:
			printf("unknown\n");
			break;
		}
	    } else {
		printf("E1\n");
	    }
	    printf("  - Line Code:\t\t");
	    switch(conf->lcode) {
		case PC300_LC_AMI:
		    printf("AMI\n");
		    break;
		case PC300_LC_B8ZS:
		    printf("B8ZS\n");
		    break;
		case PC300_LC_NRZ:
		    printf("NRZ\n");
		    break;
		case PC300_LC_HDB3:
		    printf("HDB3\n");
		    break;
		default:
		    printf("unknown\n");
		    break;
	    }
	    printf("  - Frame Mode:\t\t");
	    switch (conf->fr_mode) {
		case PC300_FR_D4:
		    printf("D4\n");
		    break;
		case PC300_FR_ESF:
		    printf("ESF\n");
		    break;
		case PC300_FR_MF_CRC4:
		    printf("CRC4\n");
		    break;
		case PC300_FR_MF_NON_CRC4:
		    printf("Non-CRC4\n");
		    break;
		case PC300_FR_UNFRAMED:
		    printf("Unframed\n");
		    break;
		default:
		    printf("unknown\n");
		    break;
	    }
	    printf("  - RX Sensitivity:\t");
	    switch(conf->rx_sens) {
		case PC300_RX_SENS_SH:
		    printf("Short Haul\n");
		    break;
		case PC300_RX_SENS_LH:
		    printf("Long Haul\n");
		    break;
		default:
		    printf("unknown\n");
		    break;
	    }
	    printf("  - Active channels:\t");
	    if(conf->tslot_bitmap == 0xffffffff) {
		printf("all\n");
	    } else {
		int i, j;
		int ch_range = ((conf->media == LINE_T1) ? 24 : 32);

		for (i = 0, j = 0; i < ch_range ; i++) {
		    if (conf->tslot_bitmap & (1<<i)) {
			printf("%d, ", i+1);
			j++;
		    }
		}
		printf("\b\b (%d channels)\n", j);
	    }
	    break;
	default:
	    printf("unknown\n");
	    break;
    }

    printf("- Protocol:\t\t");
    switch(conf->proto & ~MODE_SOFT) {
#ifdef PC300CFG_X25
	case MODE_X25:
	    printf("X.25\n");
	    break;
#endif
	case MODE_PPP:
	    printf("PPP\n");
	    break;
	case MODE_CISCO:
	    printf("Cisco HDLC\n");
	    break;
	case MODE_FR_ANSI:
	    printf("Frame Relay (ANSI)\n");
	    break;
	case MODE_FR_CCITT:
	    printf("Frame Relay (CCITT)\n");
	    break;
	case (MODE_FR_ANSI|MODE_DCE):
	    printf("Frame Relay (ANSI, DCE)\n");
	    break;
	case (MODE_FR_CCITT|MODE_DCE):
	    printf("Frame Relay (CCITT, DCE)\n");
	    break;
	case MODE_HDLC:
	    if (conf->monitor)
		printf("Monitor Mode\n");
	    else
		printf("Raw HDLC\n");
	    break;
	default:
	    printf("unknown\n");
	    break;
    }
    switch(conf->media) {
	case LINE_T1:
	case LINE_E1:
	    printf("- Clock:\t\t");
	    if (conf->clkrate) {
		printf("master\n");
	    } else {
		printf("slave\n");
	    }
	    break;

	case LINE_RS232:
	case LINE_V35:
	case LINE_X21:
	default:
	    printf("- Clock:\t\t");
	    if (conf->clkrate) {
		printf("internal (%ld bps)\n", conf->clkrate);
	    } else {
		printf("external\n");
	    }
	    break;
    }
}

static void
pc300_set_if(char *cpc_name, pc300chconf_t *conf)
{
    struct ifreq ifr;

    strncpy(ifr.ifr_name, cpc_name, IFNAMSIZ);
    ifr.ifr_data = (caddr_t) conf;
    if (ioctl(skfd, SIOCSPC300CONF, &ifr) < 0) {
	printf("ioctl err\n");
	(void) close(skfd);
	exit (-1);
    }
}

#define	IN_RANGE(val, minval, maxval)	\
		(((val) >= (minval)) && ((val) <= (maxval)))
static int
pc300_set_tslots(pc300chconf_t *conf, char *tslot_str)
{
    int i, j, is_range, tsloti, tslotf;
    char tmpstr[10], tmpstr2[10];
    int ch_range = ((conf->media == LINE_T1) ? 24 : 31);

    conf->tslot_bitmap = 0;

    for (i = 0 , is_range = 0 ; tslot_str[i] != '\0' ; i+=j) {
	for (j = 0 ; tslot_str[i+j] != '\0' ; j++) {
	    if (tslot_str[i+j] == ',') {
		i++;
		break;
	    } else if (tslot_str[i+j] == '-') {
		tmpstr[j] = '\0';
		i += j;
		j = 0;
		is_range = 1;
	    } else if (is_range) {
		tmpstr2[j - 1] = tslot_str[i+j];
	    } else {
		tmpstr[j] = tslot_str[i+j];
	    }
	}
	if (is_range) {
	    is_range = 0;
	    tmpstr2[j - 1] = '\0';
	} else {
	    tmpstr[j] = '\0';
	    strcpy(tmpstr2, tmpstr);
	}
	for (tsloti = atoi(tmpstr), tslotf = atoi(tmpstr2) ;
	     tsloti <= tslotf ; 
	     tsloti++) {
	    if (IN_RANGE(tsloti, 1, ch_range)) {
		conf->tslot_bitmap |= 1 << (tsloti - 1);
	    } else {
		return (-1);
	    }
	}
    }
    return 0;
}

int main(int argc, char **argv)
{
    char *ifname;
    pc300conf_t pc300conf;
    pc300chconf_t *conf = &pc300conf.conf;
    struct ifreq ifr;
    int i;

    if (argc < 2) {
	printf("You must specify the interface (e.g. pc300cfg hdlc0).\n");
	exit(1);
    }

    ifname = argv[1];
    memset(&pc300conf, 0, sizeof(pc300conf_t));

    /* Create a channel to the NET kernel. */
    if ((skfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
	printf("socket err\n");
	exit(1);
    }

    strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
    ifr.ifr_data = (caddr_t) &pc300conf;
    if (ioctl(skfd, SIOCGPC300CONF, &ifr) < 0) {
	printf("ioctl err\n");
	(void) close(skfd);
	exit(-1);
    }
    
    if (argc < 3) {
	pc300_show_if(ifname, conf); 
    } else {
	for (i = 2 ; i < argc ; i += 2) {
	    if (strcmp(argv[i], "media") == 0) {
		if (strcmp(argv[i+1], "rs232") == 0) {
		    conf->media = LINE_RS232;
		} else if (strcmp(argv[i+1], "v35") == 0) {
		    conf->media = LINE_V35;
		} else if (strcmp(argv[i+1], "x21") == 0) {
		    conf->media = LINE_X21;
		} else if (strcmp(argv[i+1], "t1") == 0) {
		    conf->media = LINE_T1;
		} else if (strcmp(argv[i+1], "e1") == 0) {
		    conf->media = LINE_E1;
		} else {
		    printf("Unknown media option: %s.\n", argv[i+1]);
		    exit(1);
		}
	    } else if (strcmp(argv[i], "proto") == 0) {
		if (strcmp(argv[i+1], "ppp") == 0) {
		    conf->proto = MODE_PPP|MODE_SOFT;
		    conf->monitor = 0;
		} else if (strcmp(argv[i+1], "cisco") == 0) {
		    conf->proto = MODE_CISCO|MODE_SOFT;
		    conf->monitor = 0;
		} else if (strcmp(argv[i+1], "fr_ansi") == 0) {
		    conf->proto = MODE_FR_ANSI|MODE_SOFT;
		    conf->monitor = 0;
		} else if (strcmp(argv[i+1], "fr_ccitt") == 0) {
		    conf->proto = MODE_FR_CCITT|MODE_SOFT;
		    conf->monitor = 0;
		} else if (strcmp(argv[i+1], "monitor") == 0) {
		    conf->proto = MODE_HDLC;
		    conf->monitor = 1;
#ifdef PC300CFG_X25
		} else if (strcmp(argv[i+1], "x25") == 0) {
		    conf->proto = MODE_X25|MODE_SOFT;
		    conf->monitor = 0;
#endif
		} else {
		    printf("Unknown proto option: %s.\n", argv[i+1]);
		    exit(1);
		}
	    } else if (strcmp(argv[i], "clock") == 0) {
		if (strcmp(argv[i+1], "ext") == 0) {
		    conf->clkrate = 0;
		} else {
		    conf->clkrate = atol(argv[i+1]);
		}
	    } else if (strcmp(argv[i], "lcode") == 0) {
		if (strcmp(argv[i+1], "ami") == 0) {
		    conf->lcode = PC300_LC_AMI;
		} else if (strcmp(argv[i+1], "b8zs") == 0) {
		    conf->lcode = PC300_LC_B8ZS;
		} else if (strcmp(argv[i+1], "nrz") == 0) {
		    conf->lcode = PC300_LC_NRZ;
		} else if (strcmp(argv[i+1], "hdb3") == 0) {
		    conf->lcode = PC300_LC_HDB3;
		} else {
		    printf("Unknown lcode option: %s.\n", argv[i+1]);
		    exit(1);
		}
	    } else if (strcmp(argv[i], "frame") == 0) {
		if (strcmp(argv[i+1], "d4") == 0) {
		    conf->fr_mode = PC300_FR_D4;
		} else if (strcmp(argv[i+1], "esf") == 0) {
		    conf->fr_mode = PC300_FR_ESF;
		} else if (strcmp(argv[i+1], "crc4") == 0) {
		    conf->fr_mode = PC300_FR_MF_CRC4;
		} else if (strcmp(argv[i+1], "non-crc4") == 0) {
		    conf->fr_mode = PC300_FR_MF_NON_CRC4;
		} else if (strcmp(argv[i+1], "unframed") == 0) {
		    conf->fr_mode = PC300_FR_UNFRAMED;
		} else {
		    printf("Unknown frame option: %s.\n", argv[i+1]);
		    exit(1);
		}
	    } else if (strcmp(argv[i], "lbo") == 0) {
		if (strcmp(argv[i+1], "0") == 0) {
		    conf->lbo = PC300_LBO_0_DB;
		} else if (strcmp(argv[i+1], "7.5") == 0) {
		    conf->lbo = PC300_LBO_7_5_DB;
		} else if (strcmp(argv[i+1], "15") == 0) {
		    conf->lbo = PC300_LBO_15_DB;
		} else if (strcmp(argv[i+1], "22.5") == 0) {
		    conf->lbo = PC300_LBO_22_5_DB;
		} else {
		    printf("Unknown lbo option: %s.\n", argv[i+1]);
		    exit(1);
		}
	    } else if (strcmp(argv[i], "rx_sens") == 0) {
		if (strcmp(argv[i+1], "sh") == 0) {
		    conf->rx_sens = PC300_RX_SENS_SH;
		} else if (strcmp(argv[i+1], "lh") == 0) {
		    conf->rx_sens = PC300_RX_SENS_LH;
		} else {
		    printf("Unknown rx_sens option: %s.\n", argv[i+1]);
		    exit(1);
		}
	    } else if (strcmp(argv[i], "active_ch") == 0) {
		if (strcmp(argv[i+1], "all") == 0) {
		    conf->tslot_bitmap = 0xffffffffUL;
		} else if (pc300_set_tslots(conf, argv[i+1]) != 0) {
		    printf("Invalid channel allocation: %s.\n", argv[i+1]);
		    exit(1);
		}
	    } else {
		printf("Unknown option: %s.\n", argv[i]);
		exit(1);
	    }
	}
	pc300_set_if(ifname, conf);
    }
	
    (void) close(skfd);
    return 0;
}
