//
//  This is the support file for the 93SC56 Serial EEPROM
//
#include <stdio.h>
#include <limits.h>

#include "eeprom.h"

//  Erase/Write Enable
int
eeprom_ewen (unsigned long port, int type)
{
  int i;

  eeprom_enable (port, type);
  chip_select (port, 1);
  output_word (port, 0x13, 6, i);		// Output the EWEN command

  for (i = 0 ; i < 12 ; i++)
	 clock_tick (port);

  chip_select (port, 0);

  return (0);
}

//  Erase/Write Disable
int
eeprom_ewds (unsigned long port, int type)
{
  int i;

  eeprom_enable (port, type);
  chip_select (port, 1);
  output_word (port, 0x10, 6, i);	// Output the EWDS command

  for (i = 0 ; i < 12 ; i++)
	 clock_tick (port);

  chip_select (port, 0);

  return (0);
}

//  Read a word from the EEPROM
int
eeprom_read (unsigned long port, unsigned int addr, unsigned short *data, 
	     int type)
{
  int i;

  eeprom_enable (port, type);
  chip_select (port, 1);
  output_word (port, 0x06, 4, i);		// Output the read command
  output_word (port, addr, ADDR_BITS, i);	// Output the address bits

  *data=0;
  for (i = 15 ; i >= 0 ; i--) {
	clock_tick(port);
	if (data_in (port) != 0)
		*data |= 1<<i;
  }

  chip_select (port, 0);

  return (0);
}

int
eeprom_write (unsigned long port, unsigned int addr, unsigned short data, 
	      int type)
{
  int i;

  eeprom_enable (port, type);
  chip_select (port, 1);
  output_word (port, 0x05, 4, i);		// Output the write command
  output_word (port, addr, ADDR_BITS, i);	// Output the address bits
  output_word (port, data, 16, i);		// Output the data bits

  chip_select (port, 0);
  clock_tick (port);
  chip_select (port, 1);
  clock_tick (port);

  for (i = 0 ; i < SHRT_MAX ; i++) {
	clock_tick (port);
	if (data_in(port) == 1)
		break;
  }

  chip_select (port, 0);

  if (i == SHRT_MAX)
	return (1);
  else
	return (0);
}

int
eeprom_read_block (unsigned long port, unsigned int addr,
		   void *ptr, size_t elem_size, int n, int type)
{
  unsigned short *data;
  int size, i;

  data = (unsigned short *)ptr;
  size = (elem_size * n);
  if ((size%sizeof(data[0]))!=0)
	size = (size/sizeof(data[0])) + 1;
  else
	size = size/sizeof(data[0]);

  // Read
  for (i = 0 ; i < size ; i++)
    eeprom_read (port, addr+i, data+i, type);

  return (0);
}

int
eeprom_write_block (unsigned long port, unsigned int addr,
			 void *ptr, size_t elem_size, int n, int type)
{
  unsigned short *data, j;
  int size, i;

  data = (unsigned short *)ptr;
  size = (elem_size * n);
  if ((size%sizeof(data[0]))!=0)
	size = (size/sizeof(data[0])) + 1;
  else
	size = size/sizeof(data[0]);

  eeprom_ewen (port, type);

  // Write
  for (i = 0 ; i < size ; i++) {
	eeprom_write (port, addr+i, data[i], type);
  }

  eeprom_ewds (port, type);

  // Verify
  for (i = 0 ; i < size ; i++)
	 {
		eeprom_read (port, addr+i, &j, type);
		if (j != data[i]) {
			printf("Error in pos. 0x%x of EEPROM.\n", i);
			return (1);
		}
	 }

  return (0);
}

